<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Upgrade.php 2461 2005-04-08 16:30:39Z andrew $
*/

/**
 * @package    Max
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Table/Core.php';
require_once MAX_PATH . '/lib/max/other/common.php';
require_once MAX_PATH . '/lib/max/Admin_DA.php';
require_once MAX_PATH . '/www/admin/lib-zones.inc.php'; // Needed for the constants at the top of the file...

/**
 * A class to perform upgrades from phpAdsNew 2.0 or greater, and from previous
 * version of Max.
 */
class MAX_Admin_Upgrade
{
    var $conf;
    var $dbh;
    var $tables;
    var $upgradeFrom;
    var $upgradeTo;
    var $errors;
    
    /**
     * The class constructor method.
     */
    function MAX_Admin_Upgrade($prefix = null)
    {
        $this->conf = $GLOBALS['_MAX']['CONF'];
        if (!is_null($prefix)) {
            $this->conf['table']['prefix'] = $prefix;
        }
        // Set time limit and ignore user abort, as upgrade can take some itme
        if (!ini_get('safe_mode')) {
            @set_time_limit(600);
            @ignore_user_abort(true);
        }
        $this->dbh =& MAX_DB::singleton();
        $this->tables = MAX_Table_Core::singleton($this->conf['database']['type']);
    }
    
    /**
     * A method to determine if an older version of Max (or phpAdsNew)
     * is currently installed.
     *
     * @param string $version The version of Max currently being installed.
     * @return boolean True if a previous version of Max is installed, 
     *                 false otherwise.
     */
    function previousVersionExists($version)
    {
        $this->upgradeTo = $version;
        // Does the application variables table exists? Inore any errors
        // while executing the SQL to find the version that is installed
        PEAR::pushErrorHandling(null);
        $data = $this->dbh->tableInfo($this->conf['table']['prefix'].'application_variable');
        PEAR::popErrorHandling();
        if (PEAR::isError($data)) {
            // Could not find the application variables table,
            // so definately need to upgrade
            return true;
        } else {
            // What is the current version of Max that is installed?
            $query = "
                SELECT
                    value AS max_version
                FROM
                    {$this->conf['table']['prefix']}application_variable
                WHERE
                    name = 'max_version'
                ";
            $row = $this->dbh->getRow($query);
            if (!PEAR::isError($row)) {
                $this->upgradeFrom = $row['max_version'];
                return $this->_compareVersions($this->upgradeTo, $this->upgradeFrom);
            }
        }
        // Versions were the same, or less, or there was an error.
        // Don't upgrade.
        return false;
    }
    
    /**
     * A method to perform the necessary upgrade steps to update Max
     * to the necessary database format.
     *
     * @return array Array of errors encountered during upgrade
     */
    function upgradeDatabase()
    {
        $this->errors = array();
        // Is the upgrageFrom variable defined?
        if (isset($this->upgradeFrom)) {
            // We are upgrading from Max after v0.1.16-beta,
            // so just do the required upgrade actions
            if ($this->_compareVersions('v0.2.0-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.0-alpha
                $this->_upgradeToTwoZeroAlpha();
            }
            if ($this->_compareVersions('v0.2.1-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.1-alpha
                $this->_upgradeToTwoOneAlpha();
            }
            if ($this->_compareVersions('v0.2.3-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.3-alpha
                $this->_upgradeToTwoThreeAlpha();
            }
            if ($this->_compareVersions('v0.2.4-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.2.4-alpha
                $this->_upgradeToTwoFourAlpha();
            }
            if ($this->_compareVersions('v0.3.00-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.3.00-alpha
                $this->_upgradeToThreeZeroAlpha();
            }
            if ($this->_compareVersions('v0.3.02-alpha', $this->upgradeFrom)) {
                // Upgrade to v0.3.02-alpha
                $this->_upgradeToThreeTwoAlpha();
            }
        } else {
            // Perfom *all* possible upgrade actions, in order
            $this->_upgradeEarly();            // Upgrade to v0.1.16-beta
            $this->_upgradeToTwoZeroAlpha();   // Upgrade to v0.2.0-alpha
            $this->_upgradeToTwoOneAlpha();    // Upgrade to v0.2.1-alpha
            $this->_upgradeToTwoThreeAlpha();  // Upgrade to v0.2.3-alpha
            $this->_upgradeToTwoFourAlpha();   // Upgrade to v0.2.4-alpha
            $this->_upgradeToThreeZeroAlpha(); // Upgrade to v0.3.00-alpha
            $this->_upgradeToThreeTwoAlpha();  // Upgrade to v0.3.02-alpha
        }
        if (count($this->errors) == 0) {
            // Always upgrade the installed version number
            $this->_upgradeInstalledVersion();
        }
        return $this->errors;
    }
    
    /**
     * A private method for comparing version numbers.
     *
     * @private
     * @param string $first The first version number.
     * @param string $second The second version number.
     * @return boolean True if the first version number is greater than the second,
     *                 false otherwise.
     */
    function _compareVersions($first, $second)
    {
        if ((!isset($first)) || (!isset($second))) {
            return false;
        }
        // Obtain the parts of the verison numbers
        if (preg_match('/(\d+)\.(\d+)\.(\d+)(?:-([a-z]+))?/', $first, $matches)) {
            $firstMajor = $matches[1];
            $firstMinor = $matches[2];
            $firstPatch = $matches[3];
            $firstType  = $matches[4];
        }
        if (preg_match('/(\d+)\.(\d+)\.(\d+)(?:-([a-z]+))?/', $second, $matches)) {
            $secondMajor = $matches[1];
            $secondMinor = $matches[2];
            $secondPatch = $matches[3];
            $secondType  = $matches[4];
        }
        // Compare the major versions
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor > $secondMajor)) {
            return true;
        }
        // Compare the minor versions
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor > $secondMinor)) {
                return true;
            }
        }
        // Compare the patch levels
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor == $secondMinor)) {
                if (isset($firstPatch) && isset($secondPatch) && ($firstPatch > $secondPatch)) {
                    return true;
                }
            }
        }
        // Compare the release types
        if (isset($firstMajor) && isset($secondMajor) && ($firstMajor == $secondMajor)) {
            if (isset($firstMinor) && isset($secondMinor) && ($firstMinor == $secondMinor)) {
                if (isset($firstPatch) && isset($secondPatch) && ($firstPatch == $secondPatch)) {
                    if (isset($firstType) && isset($secondType)) {
                        if (($secondType == 'alpha') && ($firstType != 'alpha')) {
                            return true;
                        } elseif (($secondType == 'beta') && (($firstType != 'alpha') || ($firstType != 'beta'))) {
                            return true;
                        } elseif (($secondType == 'rc') && ($firstType == 'stable')) {
                            return true;
                        }
                    }
                }
            }
        }
        // Version was the same, less
        return false;
    }
    
    /**
     * A method for seting (not updating) the (initial) version number
     *
     * @return boolean True if the version was set correctly, false otherwise.
     */
    function setInstalledVersion()
    {
        $query = "INSERT INTO {$this->conf['table']['prefix']}application_variable (name, value) VALUES ('max_version', '" . MAX_VERSION_READABLE . "')";
        if (PEAR::isError($this->dbh->query($query))) {
            return false;
        }
        return true;
    }
    
    /**
     * A private method for updating the version number
     *
     * @private
     */
    function _upgradeInstalledVersion()
    {
        $query = "UPDATE {$this->conf['table']['prefix']}application_variable SET value = '{$this->upgradeTo}' WHERE name = 'max_version'";
        $this->_runQuery($query);
    }
    
    /**
     * A private method to upgrade the database from the v0.3.01-alpha
     * format to the v0.3.02-alpha format.
     *
     * @private
     */
    function _upgradeToThreeTwoAlpha()
    {
        // Create the plugins_channel_delivery tables
        $this->tables->createTable('plugins_channel_delivery_assoc');
        $this->tables->createTable('plugins_channel_delivery_domains');
        $this->tables->createTable('plugins_channel_delivery_rules');
        
        // Add new campaign priority column
        $query = "ALTER TABLE {$this->conf['table']['prefix']}campaigns ADD COLUMN new_priority INT(11) NOT NULL DEFAULT '0' AFTER priority";
        $this->_runQuery($query);
        
        // Convert old style priorities to the new style
        $queries = array();
        $queries[] = "UPDATE {$this->conf['table']['prefix']}campaigns SET new_priority = 0 WHERE priority = 'l'";
        $queries[] = "UPDATE {$this->conf['table']['prefix']}campaigns SET new_priority = 1 WHERE priority = 'm'";
        $queries[] = "UPDATE {$this->conf['table']['prefix']}campaigns SET new_priority = 2 WHERE priority = 'h'";
        $this->_runQueries($queries);
        
        // Drop the old campaign priority column & rename the new one
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}campaigns DROP COLUMN priority";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}campaigns CHANGE COLUMN new_priority priority INT(11) NOT NULL DEFAULT '0'";
        $this->_runQueries($queries);
        
        // Fix any badly created userlog tables
        $query = "ALTER TABLE {$this->conf['table']['prefix']}userlog CHANGE COLUMN TIMESTAMP timestamp INT(11) NOT NULL DEFAULT '0'";
        $this->_runQuery($query);
    }
    
    /**
     * A private method to upgrade the database from the v0.3.00-alpha
     * format to the v0.3.01-alpha format.
     *
     * @private
     */
    function _upgradeToThreeOneAlpha()
    {
        // Add new column to the preference table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}preference ADD COLUMN default_banner_url VARCHAR(255) DEFAULT NULL";
        $this->_runQuery($query);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.4-alpha
     * format to the v0.3.00-alpha format.
     *
     * @private
     */
    function _upgradeToThreeZeroAlpha()
    {
        // Rename the config table to preference
        $query = "ALTER TABLE {$this->conf['table']['prefix']}config RENAME {$this->conf['table']['prefix']}preference";
        $this->_runQuery($query);
        
        // Alter columns in the preference table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}preference CHANGE updates_last_seen updates_last_seen TEXT";
        $this->_runQuery($query);
        
        // Drop unused columns from the preference table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP userlog_priority";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP userlog_autoclean";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP type_web_mode";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP type_web_dir";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP type_web_ftp";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference DROP type_web_url";
        $this->_runQueries($queries);
        
        // Add new columns to the preference table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference ADD COLUMN default_banner_url VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference ADD COLUMN default_banner_destination VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}preference ADD COLUMN banner_html_auto enum('t','f') DEFAULT 't'";
        $this->_runQueries($queries);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.3-alpha
     * format to the v0.2.4-alpha format.
     *
     * @private
     */
    function _upgradeToTwoFourAlpha()
    {
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP COLUMN table_border_color";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP COLUMN table_back_color";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP COLUMN table_back_color_alternative";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP COLUMN main_back_color";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD COLUMN gui_header_background_color VARCHAR(7) DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD COLUMN gui_header_foreground_color VARCHAR(7) DEFAULT ''";
        $this->_runQueries($queries);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.1-alpha
     * format to the v0.2.3-alpha format.
     *
     * @private
     */
    function _upgradeToTwoThreeAlpha()
    {
        // Fix the config table's gui_link_compact_limit field
        $query = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_link_compact_limit gui_link_compact_limit INTEGER DEFAULT '50'";
        $this->_runQuery($query);

        // Create the ad_zone_assoc table
        $this->tables->createTable('ad_zone_assoc');
        
        // Create the placement_zone_assoc table
        $this->tables->createTable('placement_zone_assoc');
        
        // Create the category table
        $this->tables->createTable('category');
        
        // Create the ad_category table
        $this->tables->createTable('ad_category_assoc');
        
        // Transfer keywords from the banners table to the category table
        $query = "SELECT bannerid, keyword FROM {$this->conf['table']['prefix']}banners WHERE keyword != ''";
        $result = $this->_runQuery($query);
        if (!PEAR::isError($result)) {
            while ($row = $result->fetchRow()) {
                $adId = $row['bannerid'];
                $keywords = split(' ', $row['keyword']);
                foreach ($keywords as $keyword) {
                    $innerQuery = "SELECT category_id FROM {$this->conf['table']['prefix']}category WHERE name = '$keyword'";
                    $innerResult = $this->_runQuery($innerQuery);
                    if (!PEAR::isError($innerResult)) {
                        if ($innerRow = $innerResult->fetchRow()) {
                            $categoryId = $innerRow['category_id'];
                        } else {
                            $categoryId = Admin_DA::addCategory(array('name' => $keyword));
                        }
                        Admin_DA::addAdCategory(array('ad_id' => $adId, 'category_id' => $categoryId));
                    }
                }
            }
        }
        
        // Transfer the contents of the 'what' column to the ad_zone_assoc and placement_zone_assoc tables
        $query = "SELECT zoneid, what FROM {$this->conf['table']['prefix']}zones WHERE what != ''";
        $result = $this->_runQuery($query);
        if (!PEAR::isError($result)) {
            while ($row = $result->fetchRow()) {
            	$zoneId = $row['zoneid'];
            	$keywords = split(',', $row['what']);
            	$newkeywords = array();
            	foreach ($keywords as $keyword) {
            		if (substr($keyword, 0, 9) == 'bannerid:') {
            			$adId = substr($keyword, 9);
    	                Admin_DA::addAdZone(array('zone_id' => $zoneId, 'ad_id' => $adId));
            		} elseif (substr($keyword, 0, 11) == 'campaignid:') {
            			$placementId = substr($keyword, 11);
    	                Admin_DA::addPlacementZone(array('zone_id' => $zoneId, 'placement_id' => $placementId));
    		        	MAX_addLinkedAdsToZone($zoneId, $placementId);
            		} else {
            			$newkeywords[] = $keyword;
            		}
            	}
            	$keywords = addslashes(implode(',',$newkeywords));            	
            	$innerQuery = "UPDATE {$this->conf['table']['prefix']}zones SET what = '$keywords' WHERE zoneid = $zoneId";
            	$innerResult = $this->_runQuery($innerQuery);
            }
        }
        
        // Always have a '0' node in ad_zone for each banner so that direct selection works.
        $query = "INSERT INTO {$this->conf['table']['prefix']}ad_zone_assoc (ad_id, zone_id) SELECT bannerid, 0 FROM {$this->conf['table']['prefix']}banners";
        $this->_runQuery($query);
        
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP COLUMN gui_link_compact_limit";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners DROP COLUMN keyword";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners DROP COLUMN priority";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}zones CHANGE what ad_selection TEXT NOT NULL DEFAULT ''";
        $this->_runQueries($queries);
    }
    
    /**
     * A private method to upgrade the database from the v0.2.0-alpha
     * format to the v0.2.1-alpha format.
     *
     * @private
     */
    function _upgradeToTwoOneAlpha()
    {
        // Add support for ad request summarising
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}data_intermediate_ad ADD COLUMN requests INTEGER UNSIGNED NOT NULL DEFAULT 0 AFTER zone_id";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}data_summary_ad_hourly ADD COLUMN requests INTEGER UNSIGNED NOT NULL DEFAULT 0 AFTER zone_id";
        $this->_runQueries($queries);
    }
    
    /**
     * A private method to upgrade the database from the v0.1.x format
     * to the v0.2.0-alpha format.
     *
     * @private
     */
    function _upgradeToTwoZeroAlpha()
    {
        // Add support for default anonymous campaigns to the config table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}config ADD COLUMN gui_campaign_anonymous ENUM('t','f') AFTER gui_show_campaign_preview";
        $this->_runQuery($query);
        
        // Create the data_intermediate_ad table
        $this->tables->createTable('data_intermediate_ad');
        
        // Create the data_intermediate_ad_connection table
        $this->tables->createTable('data_intermediate_ad_connection');
        
        // Create the data_intermediate_ad_variable_value table
        $this->tables->createTable('data_intermediate_ad_variable_value');
        
        // Create the data_raw_ad_click table
        $this->tables->createTable('data_raw_ad_click');
        
        // Create the data_raw_ad_impression table
        $this->tables->createTable('data_raw_ad_impression');
        
        // Create the data_raw_ad_request table
        $this->tables->createTable('data_raw_ad_request');
        
        // Create the data_raw_tracker_click table
        $this->tables->createTable('data_raw_tracker_click');
        
        // Create the data_raw_tracker_impression table
        $this->tables->createTable('data_raw_tracker_impression');
        
        // Create the data_raw_tracker_variable_value table
        $this->tables->createTable('data_raw_tracker_variable_value');
        
        // Create the data_summary_ad_hourly table
        $this->tables->createTable('data_summary_ad_hourly');
        
        // Create the data_summary_zone_impression_history table
        $this->tables->createTable('data_summary_zone_impression_history');
        
        // Create the log_maintenance_statistics table
        $this->tables->createTable('log_maintenance_statistics');
        
        // Increase the size of details in the userlog table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}userlog CHANGE details details MEDIUMTEXT";
        $this->_runQuery($query);
        
        // Drop the variabletype column from the variables table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}variables DROP variabletype";
        $this->_runQuery($query);
        
        // Add the is_basket_value column to the variables table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}variables ADD is_basket_value INTEGER NOT NULL DEFAULT '0'";
        $this->_runQuery($query);
    }
    
    /**
     * A private method to upgrade the database for all versions
     * prior to v0.1.16-beta. Note that in some cases, a check is
     * performed to test if the chages have already been made, as
     * the changes cannot be applied twice. In other cases, no check
     * is made, as the changes can simply be ignored by MySQL if
     * they are applied for a second time.
     *
     * @private
     */
    function _upgradeEarly()
    {
        // acls table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}acls CHANGE logical logical set('and','or') NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}acls CHANGE type type VARCHAR(16) NOT NULL DEFAULT ''";
        $this->_runQueries($queries);
        
        // adclicks table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks DROP INDEX bannerid_date";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks ADD userid VARCHAR(32) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks CHANGE host host VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks CHANGE source source VARCHAR(50) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks CHANGE country country char(2) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adclicks ADD INDEX bannerid(bannerid)";
        $this->_runQueries($queries);
        
        // adstats table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats DROP PRIMARY KEY";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats DROP KEY bannerid_day";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats ADD conversions INT(11) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats CHANGE bannerid bannerid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats CHANGE zoneid zoneid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats ADD KEY day(day)";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adstats ADD KEY bannerid(bannerid)";
        $this->_runQueries($queries);
        
        // adviews table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews DROP INDEX bannerid_date";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews ADD userid VARCHAR(32) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews CHANGE host host VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews CHANGE source source VARCHAR(50) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews CHANGE country country char(2) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}adviews ADD INDEX bannerid(bannerid)";
        $this->_runQueries($queries);

        // affiliates table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates ADD agencyid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates ADD mnemonic VARCHAR(5) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE name name VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE contact contact VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE email email VARCHAR(64) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE website website VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE username username VARCHAR(64) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE password password VARCHAR(64) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE permissions permissions MEDIUMINT(9) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}affiliates CHANGE language language VARCHAR(64) DEFAULT NULL";
        $this->_runQueries($queries);
        
        // agency table
        $this->tables->createTable('agency');
        
        // application_variable table
        $this->tables->createTable('application_variable');
        $query = "
            INSERT INTO
                {$this->conf['table']['prefix']}application_variable
                (
                    name,
                    value
                )
            VALUES
                (
                    'max_version',
                    '{$this->upgradeTo}'
                )
            ";
        $this->_runQuery($query);
        
        // banners table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE clientid campaignid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE filename filename VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE imageurl imageurl VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE htmltemplate htmltemplate TEXT NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE htmlcache htmlcache TEXT NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE target target VARCHAR(16) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE url url TEXT NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE alt alt VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE status status VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE keyword keyword VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE bannertext bannertext TEXT NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE description description VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD adserver VARCHAR(50) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE compiledlimitation compiledlimitation TEXT NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners CHANGE append append TEXT NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD alt_filename VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD alt_imageurl VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD alt_contenttype ENUM('gif','jpeg','png') NOT NULL DEFAULT 'gif'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD KEY campaignid (campaignid)";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD capping INT(11) NOT NULL DEFAULT '0' AFTER block";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}banners ADD session_capping INT(11) NOT NULL DEFAULT '0' AFTER capping";
        $this->_runQueries($queries);
        
        // cache table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}cache CHANGE content content blob NOT NULL";
        $this->_runQuery($query);
        
        // campaigns table
        $this->tables->createTable('campaigns');
        $query = "
            INSERT INTO
                {$this->conf['table']['prefix']}campaigns
            SELECT
                clientid AS campaignid,
                clientname AS campaignname,
                parent AS clientid,
                views AS views,
                clicks AS clicks,
                '-1' AS conversions,
                expire AS expire,
                activate AS activate,
                active AS active,
                'h' AS priority,
                weight AS weight,
                target AS target,
                'f' AS optimise,
                'f' AS anonymous
            FROM
                {$this->conf['table']['prefix']}clients
            WHERE
                parent > 0
        ";
        // There may be an error copying the data from the old phpAdsNew
        // clients table into the new Max campaigns table, because this 
        // might be an upgrade from an early version of Max, in which case
        // the data has already been copied. So, don't use a PEAR_Error
        // handler at all for this query.
        PEAR::pushErrorHandling(null);
        $result = $this->dbh->query($query);
        PEAR::popErrorHandling();
        if (PEAR::isError($result)) {
            // Was this the table doesn't exist error?
            if ($result->code != DB_ERROR_NOSUCHTABLE) {
                // No, the old phpAdsNew clients table existed, so it was
                // a real error that happened
                $this->errors[] = MAX::errorObjToString($result);
            }
        } else {
            // The copy of data worked, so now update the data in the
            // campaigns table
            $query = "
                UPDATE
                    {$this->conf['table']['prefix']}campaigns
                SET
                    priority = 'l'
                WHERE
                    weight > 0
                    AND views = -1
                    AND clicks = -1
                    AND conversions = -1
                ";
            $this->dbh->query($query);
        }
        
        // campaigns_trackers table
        $this->tables->createTable('campaigns_trackers');
        
        // clients table
        $queries = array();
        $queries[] = "DELETE FROM {$this->conf['table']['prefix']}clients WHERE parent > 0";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients ADD agencyid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE clientname clientname VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE contact contact VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE email email VARCHAR(64) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE clientusername clientusername VARCHAR(64) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE clientpassword clientpassword VARCHAR(64) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE permissions permissions MEDIUMINT(9) default NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients CHANGE language language VARCHAR(64) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients DROP active";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients DROP weight";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients DROP target";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}clients DROP parent";
        $this->_runQueries($queries);
        
        // config table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config DROP PRIMARY KEY";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE configid agencyid MEDIUMINT(9) DEFAULT '0' NOT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE my_header my_header VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE my_footer my_footer VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE language language VARCHAR(32) DEFAULT 'english'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE name name VARCHAR(32) default NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE override_gd_imageformat override_gd_imageformat VARCHAR(4) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE begin_of_week begin_of_week TINYINT(2) DEFAULT '1'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE percentage_decimals percentage_decimals TINYINT(2) DEFAULT '2'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_sql_allow type_sql_allow ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_url_allow type_url_allow ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_web_allow type_web_allow ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_html_allow type_html_allow ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_txt_allow type_txt_allow ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_web_mode type_web_mode TINYINT(2) DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_web_dir type_web_dir VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_web_ftp type_web_ftp VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE type_web_url type_web_url VARCHAR(255) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin admin VARCHAR(64) DEFAULT 'phpadsuser'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin_pw admin_pw VARCHAR(64) default 'phpadspass'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin_fullname admin_fullname VARCHAR(255) DEFAULT 'Your Name'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin_email admin_email VARCHAR(64) DEFAULT 'your@email.com'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD warn_admin ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD warn_agency ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD warn_client ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD warn_limit MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin_email_headers admin_email_headers VARCHAR(64) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE admin_novice admin_novice ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE default_banner_weight default_banner_weight TINYINT(4) DEFAULT '1'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE default_campaign_weight default_campaign_weight TINYINT(4) DEFAULT '1'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE client_welcome client_welcome ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE client_welcome_msg client_welcome_msg TEXT";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE content_gzip_compression content_gzip_compression ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE userlog_email userlog_email ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE userlog_priority userlog_priority ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE userlog_autoclean userlog_autoclean ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_campaign_info gui_show_campaign_info ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_campaign_preview gui_show_campaign_preview ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_banner_info gui_show_banner_info ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_banner_preview gui_show_banner_preview ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_banner_html gui_show_banner_html ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_matching gui_show_matching ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_show_parents gui_show_parents ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE gui_hide_inactive gui_hide_inactive ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE qmail_patch qmail_patch ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE updates_frequency updates_frequency TINYINT(2) DEFAULT '7'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE updates_timestamp updates_timestamp INT(11) DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE updates_last_seen updates_last_seen decimal(7,3) DEFAULT '0.000'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_plain allow_invocation_plain ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD allow_invocation_plain_nocookies ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_js allow_invocation_js ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_frame allow_invocation_frame ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_xmlrpc allow_invocation_xmlrpc ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_local allow_invocation_local ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_interstitial allow_invocation_interstitial ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE allow_invocation_popup allow_invocation_popup ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE auto_clean_tables auto_clean_tables ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE auto_clean_tables_interval auto_clean_tables_interval TINYINT(2) DEFAULT '5'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE auto_clean_userlog auto_clean_userlog ENUM('t','f') DEFAULT 'f'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE auto_clean_userlog_interval auto_clean_userlog_interval TINYINT(2) DEFAULT '5'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE auto_clean_tables_vacuum auto_clean_tables_vacuum ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE autotarget_factor autotarget_factor float DEFAULT '-1'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config CHANGE maintenance_timestamp maintenance_timestamp INT(11) DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD compact_stats ENUM('t','f') DEFAULT 't'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD statslastday DATE NOT NULL DEFAULT '0000-00-00'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD statslasthour TINYINT(4) NOT NULL DEFAULT '0'";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}config ADD PRIMARY KEY(agencyid)";
        $this->_runQueries($queries);
        
        // conversionlog table
        $this->tables->createTable('conversionlog');
        
        // images table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}images CHANGE filename filename VARCHAR(128) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}images CHANGE t_stamp t_stamp TIMESTAMP(14) NOT NULL";
        $this->_runQueries($queries);
        
        // log_maintenance table
        $this->tables->createTable('log_maintenance');

        // session table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}session CHANGE sessionid sessionid VARCHAR(32) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}session CHANGE lastused lastused TIMESTAMP(14) NOT NULL";
        $this->_runQueries($queries);

        // Sometimes phpAdsNew installs the targetstats table without
        // the correct prefix, and instead uses "tbl_" as the prefix.
        // Run some SQL to fix this, but ignore all errors.
        PEAR::pushErrorHandling(null);
        $query = "ALTER TABLE tbl_targetstats RENAME {$this->conf['table']['prefix']}targetstats";
        $this->dbh->query($query);
        PEAR::popErrorHandling();
        
        // targetstats table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}targetstats DROP PRIMARY KEY";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}targetstats CHANGE clientid campaignid MEDIUMINT(9) NOT NULL DEFAULT '0'";
        $this->_runQueries($queries);
        
        // trackers table
        $this->tables->createTable('trackers');
        
        // userlog table
        $query = "ALTER TABLE {$this->conf['table']['prefix']}userlog CHANGE details details TEXT";
        $this->dbh->query($query);
        
        // variables table
        $this->tables->createTable('variables');
        
        // variablevalues table
        $this->tables->createTable('variablevalues');
        
        // zones table
        $queries = array();
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}zones CHANGE affiliateid affiliateid MEDIUMINT(9) DEFAULT NULL";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}zones CHANGE zonename zonename VARCHAR(245) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}zones CHANGE description description VARCHAR(255) NOT NULL DEFAULT ''";
        $queries[] = "ALTER TABLE {$this->conf['table']['prefix']}zones ADD COLUMN forceappend ENUM ('t','f') DEFAULT 'f'";
        $this->_runQueries($queries);
        
        // Fix the 'what' column which links campaigns (used to be clients)
        $query = "
            SELECT
                *
            FROM
                {$this->conf['table']['prefix']}zones
            ";
        $result = $this->dbh->query($query);
        if (!PEAR::isError($result)) {
            while ($row = $result->fetchRow()) {
                $newWhat = preg_replace('/clientid:/', 'campaignid:', $row['what']);
                if ($newWhat!= $row['what']) {
                    $innerQuery = "
                        UPDATE
                            {$this->conf['table']['prefix']}zones
                        SET
                            what = '$newWhat'
                        WHERE
                        ";
                    foreach ($row as $key => $value) {
                        if (preg_match('/\d+/', $key)) {
                            continue;
                        } elseif ($key == 'what') {
                            continue;
                        } elseif ($value != '') {
                            $innerQuery .= "$key = '$value' AND ";
                        }
                    }
                    $innerQuery = preg_replace('/ AND $/', '', $innerQuery);
                    if (!preg_match('/WHERE $/', $innerQuery)) {
                        $this->dbh->query($innerQuery);
                    }
                }
            }
        }
    }
    
    /**
     * A private method for running a query and returning the result.
     *
     * @private
     * @param string $query A SQL query to run.
     * @return mixed A {@link DB_Result} or {@link DB_Error} object.
     */
    function _runQuery($query)
    {
        // Use the Upgrade class' PEAR_Error handler
        PEAR::pushErrorHandling(PEAR_ERROR_CALLBACK, array($this, 'pearErrorHandler'));
        // Execute the upgrade SQL statements
        $result = $this->dbh->query($query);
        // Restore the normal PEAR_Error handler
        PEAR::popErrorHandling();
        // Return the result
        return $result;
    }
    
    /**
     * A private method for running a series of queries.
     *
     * @private
     * @param array $queries An array of SQL queries to run.
     */
    function _runQueries($queries)
    {
        // Use the Upgrade class' PEAR_Error handler
        PEAR::pushErrorHandling(PEAR_ERROR_CALLBACK, array($this, 'pearErrorHandler'));
        // Execute the upgrade SQL statements
        foreach ($queries as $query) {
            $this->dbh->query($query);
        }
        // Restore the normal PEAR_Error handler
        PEAR::popErrorHandling();
    }

    /**
     * A callback method for setting the default PEAR_Error handling behaviour
     * when running the upgrade SQL statements.
     *
     * @param PEAR_Error $oError A {@link PEAR_Error} object.
     */
    function pearErrorHandler($oError)
    {
        if (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+(?!INDEX)(?!KEY)(?!PRIMARY\s+KEY)/', $oError->userinfo)) {
            // May caused by trying to re-add a column, if so, ignore
            if ($oError->code == DB_ERROR) {
                return;
            }
        } elseif (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+(INDEX|KEY)/', $oError->userinfo)) {
            // May caused by trying to re-add an index or key, if so, ignore
            if ($oError->code == DB_ERROR_ALREADY_EXISTS) {
                return;
            }
        } elseif (preg_match('/^ALTER\s+TABLE\s+\w+\s+ADD\s+PRIMARY\s+KEY/', $oError->userinfo)) {
            // May caused by trying to re-add a primary key, if so, ignore
            if ($oError->code == DB_ERROR_ALREADY_EXISTS) {
                return;
            }
        } elseif (preg_match('/^ALTER\s+TABLE\s+\w+\s+CHANGE/', $oError->userinfo)) {
            // May caused by trying to change a non-existant column, if so, ignore
            if ($oError->code == DB_ERROR_NOSUCHFIELD) {
                return;
            }
        } elseif (preg_match('/^ALTER\s+TABLE\s+\w+\s+DROP(?!PRIMARY\s+KEY)/', $oError->userinfo)) {
            // May caused by trying to drop a non-existant index, key, or column, if so, ignore
            if ($oError->code == DB_ERROR) {
                return;
            }
        } elseif (preg_match('/^DELETE\s+FROM/', $oError->userinfo)) {
            // May caused by tring to delete where a column has already been dropped, if so, ignore
            if ($oError->code == DB_ERROR_CANNOT_DELETE) {
                return;
            }
        }
        // This is a real error - add it to the array of errors
        $this->errors[] = MAX::errorObjToString($oError);
    }
    
}

?>
