<?php
/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: InvocationTags.php 2280 2005-03-30 14:05:56Z radek $
*/

define('MAX_PLUGINS_INVOCATION_TAGS_ALLOW', 'Allow ');

define('MAX_PLUGINS_INVOCATION_TAGS_STANDARD', 0);
define('MAX_PLUGINS_INVOCATION_TAGS_CUSTOM', 1);

/**
 * @package    MaxPlugins
 * @subpackage InvocationTags
 * @author     Radek Maciaszek <radek@awarez.net>
 */

require_once MAX_PATH . '/lib/max/Plugins/CommonPlugin.php';

/**
 *
 * Plugins_InvocationTags is an abstract class for every Invocation tag plugin
 *
 */
class Plugins_InvocationTags extends Max_CommonPlugin 
{
    /**
     * With the help of this variable we could
     * pass the globals variables from
     * Max_Invocation object in more object oriented way - as
     * object attributes
     */
    var $maxInvocation;
    
    /**
     * Return name of plugin
     *
     * @return string A string describing the class.
     */
    function getName()
    {
        Max::debug('Cannot run abstract method');
        exit();
    }
    
    /**
     * Return preference code - required for preferences
     *
     * @return string A string describing the class.
     */
    function getPreferenceCode()
    {
        Max::debug('Cannot run abstract method');
        exit();
    }

    /**
     * Return list of options
     * generateOptions() use this information to generate the HTML FORM
     * containing configuration options
     * 
     * @see generateOptions()
     * @return array    Array of options names. Key is option name and value is option type
     *                  Option type could be:
     *                    - MAX_PLUGINS_INVOCATION_TAGS_STANDARD - option name is a method
     *                                                             from Plugins_InvocationTagsOptions classs
     *                    - MAX_PLUGINS_INVOCATION_TAGS_CUSTOM - option name is name of the method
     *                                                           from plugin class
     */
    function getOptionsList()
    {
        Max::debug('Cannot run abstract method');
        exit();
    }

    /**
     * Return translation: "allow _invocation tag name_"
     *
     * @return string    Translated string
     */
    function getAllowInvocationTypeForSettings()
    {
    	include_once MAX_PATH . '/lib/max/Plugins/PluginsTranslation.php';
    	return Max_PluginTranslation::translate(MAX_PLUGINS_INVOCATION_TAGS_ALLOW . $this->getName(), $this->module, $this->package);
    }
    
    /**
     * Check if current plugin is allowed in preferences
     *
     * @param array    Extra parameters which could be used in child classes
     *
     * @return boolean    True if allowed else false
     */
    function isAllowed($extra = null)
    {
        $pref = $GLOBALS['_MAX']['PREF'];
        $invocationCode = $this->getPreferenceCode();
        return isset($pref[$invocationCode]) ? $pref[$invocationCode] : false;
    }
    
    /**
     * Inject invocation - required for generateInvocationCode()
     * and for custom options methods
     *
     * @see generateInvocationCode()
     */
    function setInvocation(&$invocation) {
        $this->maxInvocation = &$invocation;
    }
    
    /**
     * Generate the invocation string
     *
     * @return string    Generated invocation string
     */
    function generateInvocationCode()
    {
        Max::debug('Cannot run abstract method');
        exit();
    }
    
    /**
     * Prepare data before generating the invocation code
     *
     */
    function prepareCommonInvocationData()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $mi = &$this->maxInvocation;
        
        // Check if affiliate is on the same server
        if (isset($mi->website) && $mi->website != '') {
            $mi->server_max      = parse_url('http://' . $conf['webpath']['delivery'] . '/');
            $mi->server_affilate = parse_url($mi->website);
            $mi->server_same     = (@gethostbyname($mi->server_max['host']) == @gethostbyname($mi->server_affilate['host']));
        } else {
            $mi->server_same = true;
        }
    
        $mi->parameters = array();
        $mi->uniqueid = 'a'.substr(md5(uniqid('', 1)), 0, 7);
        if (!isset($mi->withtext)) {
            $mi->withtext = 0;
        }
    
        // Set parameters
        if (isset($mi->clientid) && strlen($mi->clientid) && $mi->clientid != '0') {
            $mi->parameters['clientid'] = "clientid=".$mi->clientid;
        }
        if (isset($mi->zoneid) && $mi->zoneid != '') {
            $mi->parameters['zoneid'] = "zoneid=".urlencode($mi->zoneid);
        }
        if (isset($mi->campaignid) && strlen($mi->campaignid) && $mi->campaignid != '0') {
            $mi->parameters['campaignid'] = "campaignid=".$mi->campaignid;
        }
        if (isset($mi->bannerid) && $mi->bannerid != '') {
            $mi->parameters['bannerid'] = "bannerid=".urlencode($mi->bannerid);
        }
        if (isset($mi->what) && $mi->what != '') {
            $mi->parameters['what'] = "what=".str_replace (",+", ",_", $mi->what);
        }
        if (isset($mi->source) && $mi->source != '') {
            $mi->parameters['source'] = "source=".urlencode($mi->source);
        }
        if (isset($mi->target) && $mi->target != '') {
            $mi->parameters['target'] = "target=".urlencode($mi->target);
        }
    }
    
    /**
     * Generate all the options for plugin settings and return as HTML
     *
     * @param object $maxInvocation    Max_Invocation object
     *
     * @return string
     */
    function generateOptions(&$maxInvocation)
    {
        $this->setInvocation($maxInvocation);
        
        $show = $this->getOptionsList($maxInvocation);
        
        include_once MAX_PATH . '/plugins/invocationTags/InvocationTagsOptions.php';
        $invocationOptions = new Plugins_InvocationTagsOptions();
        $invocationOptions->setInvocation($maxInvocation);
        
        $htmlOptions = '';
        foreach($show as $optionToShow => $methodType) {
            if($methodType == MAX_PLUGINS_INVOCATION_TAGS_STANDARD) {
                if(!method_exists($invocationOptions, $optionToShow)) {
                    MAX::raiseError("Method '$optionToShow' doesn't exists");
                } else {
                    $htmlOptions .= $invocationOptions->$optionToShow();
                }
            } else {
                if(!method_exists($this, $optionToShow)) {
                    MAX::raiseError("Method '$optionToShow' doesn't exists");
                } else {
                    $htmlOptions .= $this->$optionToShow();
                }
            }
        }
        
        return $htmlOptions;
    }
    
}

?>