#!/usr/bin/php -q
<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| Copyright (c) 2000-2003 the phpAdsNew developers                          |
| For contact details, see: http://www.phpadsnew.com/                       |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: maintenance.php 2153 2005-03-15 17:59:45Z andrew $
*/

/**
 * A script file to run the Maintenance process.
 */

// Require the initialisation file
$path = dirname(__FILE__);
require_once $path . '/../../init.php';

// Required files
require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once MAX_PATH . '/lib/max/Maintenance/Statistics.php';
require_once MAX_PATH . '/scripts/maintenance/maintenance-priority.php';

// Set longer time out, and ignore user abort
if (!ini_get('safe_mode')) {
    @set_time_limit(300);
    @ignore_user_abort(true);
}

// Record the Maintenance Engine start time
$maintenanceStartTime = time();

MAX::debug('Running Maintenance', PEAR_LOG_INFO);

// Get a connection to the datbase
$dbh =& MAX_DB::singleton();

// Check the operation interval is valid
$result = Maintenance::checkOperationIntervalValue($conf['maintenance']['operationInterval']);
if (PEAR::isError($result)) {
    // Unable to continue!
    MAX::raiseError($result, null, PEAR_ERROR_DIE);
}

// If split tables, check for lockfile
if ($conf['table']['split']) {
    MAX::debug('Checking for lockfile', PEAR_LOG_INFO);
    $attempt = 0;
    // Don't start if lockfile detected
    while (file_exists($conf['table']['lockfile'])) {
        if ($attempt > 2) {
            // Give up on maintenance
            MAX::debug('More than 3 attempts, sending email to admin user', PEAR_LOG_ERR);
            $message  = "Warning! Maintenance was unable to run - the lockfile was found\n";
            $message .= "in place while trying to start maintenance, even after 3 iterations.\n\n";
            $message .= "The lockfile used was: {$conf['table']['lockfile']}.\n\n";
            $query = "
                SELECT
                    admin_email
                FROM
                    {$conf['table']['prefix']}{$conf['table']['preference']}
                WHERE
                    agencyid = 0
                ";
            $row = $dbh->getRow($query);
            MAX::sendMail($row['admin_email'], '', 'Lockfile altert!', $message);
            MAX::raiseError('Aborting script execution', null, PEAR_ERROR_DIE);
        }
        // Pause for 30 secs
        MAX::debug('Lockfile exists, sleeping for 30 secs. Iteration ' . $attempt, PEAR_LOG_INFO);
        sleep(30);
        $attempt ++;
    }
    // Write lockfile so table splitting scripts will abort if they
    // attempt to start during a maintenance run
    MAX::debug('No lockfile exists, writing lockfile', PEAR_LOG_INFO);
    $fh = fopen($conf['table']['lockfile'], 'w');
}

// Run Maintenance Statistics
MAX::debug('Running Maintenance Statistics', PEAR_LOG_INFO);
if ($conf['table']['split']) {
    // Disable error handling, as may have table errors
    PEAR::pushErrorHandling(null);
}
foreach ($conf['modules'] as $module => $installed) {
    if ($installed) {
        // Create an appropriate Maintenance_Statistics class
        $maintenanceStatistics = Maintenance_Statistics::instance($module);
        // Step 1: Summarise raw data into the intermediate tables
        $maintenanceStatistics->summariseIntermediate();
        // Step 2: Summarise intermediate data into summary tables
        $maintenanceStatistics->summariseFinal();
        // If the module is the AdServer module...
        if ($module == 'AdServer') {
            // Optional Step 2.5: Decrement the campaign limitations,
            // and activate/deactivate campaigns
            $maintenanceStatistics->decrementCampaigns();
        }
        // Step 3: Delete old raw data that has been summarised
        $maintenanceStatistics->deleteOldData();
        // Step 4: Update the log tables detailing the maintenance statistics run
        $maintenanceStatistics->logCompletion();
    }
}
if ($conf['table']['split']) {
    // Re-enable error handling
    PEAR::popErrorHandling();
}
MAX::debug('Maintenance Statistics Completed', PEAR_LOG_INFO);

// Disable magic_quotes_runtime
// set_magic_quotes_runtime(0);

// Translation screens
// require_once MAX_PATH. '/scripts/maintenance/translationStrings.php';

// Run Midnight phpAdsNew Tasks
//if (date('H') == 0) {
    MAX::debug('Running Midnight Maintenance Tasks', PEAR_LOG_INFO);
    include_once MAX_PATH . '/scripts/maintenance/maintenance-reports.php';
    include_once MAX_PATH . '/scripts/maintenance/maintenance-autotargeting.php';
    include_once MAX_PATH . '/scripts/maintenance/maintenance-optimization.php';
    MAX::debug('Midnight Maintenance Tasks Completed', PEAR_LOG_INFO);
//}

// Run phpAdsNew Maintenance Priority
MAX::debug('Running Maintenance Priority', PEAR_LOG_INFO);
phpAds_PriorityCalculate();
MAX::debug('Maintenance Priority Completed', PEAR_LOG_INFO);

// Remove lockfile, if required
if ($conf['table']['split']) {
    MAX::debug('Removing lockfile', PEAR_LOG_INFO);
    unlink($conf['table']['lockfile']);
}

// Record the maintenance end time
$maintenanceEndTime = time();

// Update the timestamp (for old maintenance code)
MAX::debug('Updating the timestamp in the preference table', PEAR_LOG_DEBUG);
$query = "
    UPDATE
        {$conf['table']['prefix']}{$conf['table']['preference']}
    SET
        maintenance_timestamp = UNIX_TIMESTAMP(NOW())";
$result = $dbh->query($query);

// Log the maintenance run
MAX::debug('Logging the run in the log_maintenance table', PEAR_LOG_DEBUG);
$elapsedTime = $maintenanceEndTime - $maintenanceStartTime;
$maintenanceStartTime = date("Y-m-d H:i:s", $maintenanceStartTime);
$maintenanceEndTime   = date("Y-m-d H:i:s", $maintenanceEndTime);
$query = "
    INSERT INTO
        {$conf['table']['prefix']}{$conf['table']['log_maintenance']}
        (
            start_run,
            end_run,
            duration
        )
    VALUES
        (
            '$maintenanceStartTime',
            '$maintenanceEndTime',
            $elapsedTime
        )";
$result = $dbh->query($query);

MAX::debug('Maintenance Completed', PEAR_LOG_INFO);
    
?>
